type TExercise
  as string title          ' displayed on the button (main window)
  as string id             ' identification for exercise.dat
  as string listname       ' used for the name of the save file
  ' listname can be "full", but then it shouldn't change anything in the word positions
  as string settingString  ' user preferences; e. g. word groups that should be used
  as string menu           ' menu in which the entry has to appear (e. g. "main", "games", "grammar")
  as string headline       ' headline, e. g. for menus
  as long opt              ' for free use (in categories, opt is used to specify MENUTYPE)
  as TList fullList        ' all words used for the exercise
  as TList currentList     ' words currently in use
  as function(as TExercise) as long callback                  ' main function of the exercise
  as function(as TExercise) as string info                    ' info string displayed in the main window
  as function(as TExercise, as long = 1) as long settings     ' function to set the preferences
  ' In the beginnig SETTINGSTRING is read from file, and SETTINGS(exercise, 0) is called
  ' to allow the function to initialize (especially to execute SETTINGSTRING and set CURRENTLIST).
  ' The main program will call SETTINGS(exercise, 1) to enter the setting window.
  ' You may call the function from CALLBACK and choose a different parameter to disable some options.
  declare constructor
  declare constructor(id as string, title as string, listname as string, opt as long, _
          callback as function(as TExercise) as long, _
          info as function(as TExercise) as string, _
          settings as function(as TExercise, as long = 1) as long, _
          headline as string = "", menu as string = "main")
  declare operator cast as string
  declare sub updateLists
  declare function exportString as string
  declare sub importString(settings as string)
end type

constructor TExercise
end constructor

constructor TExercise(i as string, t as string, l as string, o as long, c as function(as TExercise) as long, _
                      h as function(as TExercise) as string, s as function(as TExercise, as long = 1) as long, _
                      hl as string = "", m as string = "main")
  id       = i
  title    = t
  listname = l
  fulllist.listname = l
  opt      = o
  callback = c
  info     = h
  settings = s
  headline = iif(len(hl), hl, title)
  menu     = m
end constructor

operator =(e1 as TExercise, e2 as TExercise) as integer
  return e1.id = e2.id
end operator

operator TExercise.cast as string
  return title
end operator

sub TExercise.updateLists
  fullList.update
  dim as TListPoint listPoint, lp
  dim as TList tempList
  dim as integer i
  tempList.list.addAll(currentList.list)
  currentList.list.clear
  currentList.size = 0
  currentList.notChecked = 0
  foreach(TListPoint, listPoint, fullList.list)
    if tempList.list.contains(listPoint) then
      currentList.list.add(listPoint)
      currentList.size += 1
    end if
  nexteach
  currentList.sort
  currentList.update
end sub

function TExercise.exportString as string
  ' returns string that can be saved into exercise.dat
  ' contains:
  ' * length of id (as long)
  ' * id
  ' * length of settingString (as longint)
  ' * settingString
  return mkl(len(id)) & id & mklongint(len(settingString)) & settingString
end function

sub TExercise.importString(settings as string)
  if len(settings) < 4 then exit sub
  dim as integer settingLength, idLength = cvl(left(settings, 4))
  dim as string storedID = mid(settings, 5, idLength)
  if storedID <> id or len(settings) < idLength+12 then exit sub
  settingLength = cvlongint(mid(settings, idLength+5, 8))
  settingString = mid(settings, idLength+13)
end sub
