type TWordPosition
  as integer x, w
  as integer size, special   ' not implemented yet
  as string text
  declare operator cast as string
end type

operator TWordPosition.cast as string
  return mki(x) & mki(w) & mki(len(text)) & text
end operator
operator =(w1 as TWordPosition, w2 as TWordPosition) as integer
  return str(w1) = str(w2)
end operator
mdListDeclare(TWordPosition)

type TTextLine
  as integer size, w
  as FT_FaceFam font = font
  as mdList(TWordPosition) words
  declare operator cast as string
end type

operator TTextLine.cast as string
  return ""
end operator
operator =(w1 as TTextLine, w2 as TTextLine) as integer
  return str(w1) = str(w2)
end operator
mdListDeclare(TTextLine)

type TTextBlock
  as FB.Image ptr img
  as mdList(TTextLine) lines
  declare destructor
  declare sub add(w as integer, byref t as string, font as FT_FaceFam, size as integer = FONTSIZE.NORMAL)
  declare sub getImg
  declare sub show(x as integer, y as integer, byval h as uinteger = 0, byval posY as uinteger = 0)
end type

destructor TTextBlock
  if img then imagedestroy img
end destructor

sub TTextBlock.add(w as integer, byref t as string, font as FT_FaceFam, size as integer = FONTSIZE.NORMAL)
  dim as integer search = instr(t, "<br />")
  do while search
    add w, left(t, search-1), font, size
    t = mid(t, search+6)
    search = instr(t, "<br />")
  loop

  dim as integer position = 1, spaceWidth = FTsize(" ", font, size).x
  dim as integer tempInt, wordWidth, lineWidth = 0, curFontFace = FONTFACE.NORMAL
  dim as string currentWord
  dim as TTextLine currentLine
  dim as TWordPosition tempWord
  dim as single wordSpace
  currentLine.font = font
  currentLine.size = size
  currentLine.w    = w

  do
    ' extract next word
    search = instr(position+1, t, any chr(9, 32))
    if search then
      currentWord = mid(t, position, search-position)
      do
        search += 1
      loop until search >= len(t) orelse t[search-1] <> 9 and t[search-1] <> 32
      position = search
    else
      currentWord = mid(t, position)
      position = len(t) + 1
    end if
    ' add old face informations
    if curfontFace and FONTFACE.BOLD      then currentWord = chr(27, 66) & currentWord
    if curfontFace and FONTFACE.SMALLCAPS then currentWord = chr(27, 67) & currentWord
    if curfontFace and FONTFACE.ITALIC    then currentWord = chr(27, 73) & currentWord
    ' search escape sequences
    tempInt = 0
    do while tempInt < len(currentWord)
      if currentWord[tempInt] = 27 then
        tempInt += 1
        if tempInt >= len(currentWord) then exit do
        select case currentWord[tempInt]
          case 78, 110  ' N, n
            curFontFace = FONTFACE.NORMAL
          case 66       ' B
            curFontFace or= FONTFACE.BOLD
          case 67       ' C
            curFontFace or= FONTFACE.SMALLCAPS
          case 73       ' I
            curFontFace or= FONTFACE.ITALIC
          case 98       ' b
            curFontFace xor= FONTFACE.BOLD
          case 99       ' c
            curFontFace xor= FONTFACE.SMALLCAPS
          case 105      ' i
            curFontFace xor= FONTFACE.ITALIC
        end select
      end if
      tempInt += 1
    loop
    wordWidth = FTsize(currentWord, font, size).x
    if lineWidth + wordWidth > w then
      wordSpace = (w-lineWidth-spaceWidth) / (currentLine.words.size-1)
      tempWord.text = ""
      currentLine.words.add(tempword)
      for i as integer = 1 to currentLine.words.size-1
        tempWord = currentLine.words.get(i)
        tempWord.x += i*wordSpace
        currentLine.words.set(i, tempword)
      next
      lines.add currentLine
      currentLine.words.clear
      tempWord.text = currentWord
      tempWord.x = 0
      currentLine.words.add tempWord
      lineWidth = wordWidth + spaceWidth
    else
      tempWord.text = currentWord
      tempWord.x = lineWidth
      currentLine.words.add tempWord
      lineWidth += wordWidth + spaceWidth
    end if
  loop while position < len(t)
  lines.add currentLine
end sub

sub TTextBlock.getImg
  dim as TTextLine currentLine
  dim as TWordPosition currentWord
  dim as integer spaceHeight, blockWidth = 0, blockHeight = 0, posY = 0
  for i as integer = 0 to lines.size-1
    currentLine = lines.get(i)
    blockHeight += FTsize(" ", currentLine.font, currentLine.size).y
    if blockWidth < currentLine.w then blockWidth = currentLine.w
  next
  if img then imagedestroy img
  img = imagecreate(blockWidth, blockHeight, rgb(255, 255, 255))
  for i as integer = 0 to lines.size-1
    currentLine = lines.get(i)
    spaceHeight = FTsize(" ", currentLine.font, currentLine.size).y
    for k as integer = 0 to currentLine.words.size-1
      currentWord = currentLine.words.get(k)
      printFT currentWord.x, posY, currentWord.text, currentLine.font, currentLine.size,, img
    next
    posY += spaceHeight
  next
end sub

sub TTextBlock.show(x as integer, y as integer, byval h as uinteger = 0, byval posY as uinteger = 0)
  if img = 0 then getImg
  if img = 0 then exit sub
  if h = 0 or h > img->height then h = img->height
  if posY > img->height - h then posY = img->height - h
  if h = img->height then
    put (x+5, y), img, (0, posY)-step(img->width-1, h-1), pset
  else
    put (x, y), img, (0, posY)-step(img->width-1, h-1), pset
    line (x+img->width+5, y)-step(5, h), rgb(220, 220, 220), bf
    line (x+img->width+5, y+posY*h/img->height)-step(5, h*h/img->height), rgb(160, 160, 160), bf
  end if
end sub
