' module: create a crossword puzzle

namespace CROSSWORD
  enum Direction
    rightup = 0, right, rightdown, down
    leftdown, left, leftup, up
    last
  end enum
  enum DirecOptions
    all = 0, noDiag, forwards, lastDirec = 15
    fill = 16, numbers = 32, connect = 64, spaces = 128, randomFlex = 256
  end enum
  declare function checkPosibility(char() as integer, locked() as integer, x as integer, y as integer, direc as integer, _
                                   possibilities as integer, txt as wstring ptr) as integer
  declare function findFilling(length() as integer, empty as integer, first as integer = 0) as string
  declare sub getMinMaxPos(byref xMin as integer, byref xMax as integer, byref yMin as integer, byref yMax as integer, _
                           w as integer, h as integer, size as integer, direc as integer)
  declare sub getRandomPos(byref x as integer, byref y as integer, w as integer, h as integer, size as integer, direc as integer)

  declare function callbackCrossword(exercise as TExercise) as long
  declare function callbackWordsoup(exercise as TExercise) as long
  declare sub mark(gx1 as integer, gx2 as integer, gy1 as integer, gy2 as integer, _
                             startX as integer, startY as integer, boxWidth as integer)
  declare function settings(exercise as TExercise, optional as long = SETTING_OPTIONS.NONE) as long

  type TCWord
    as ubyte x, y, direction, found
    as TWord value
    as TWString hint
    declare constructor
    declare constructor(x_ as ubyte, y_ as ubyte, d_ as ubyte, v_ as TWord)
    declare operator cast as string
  end type
  operator =(w1 as TCWord, w2 as TCWord) as integer
    return w1.x = w2.x andalso w1.y = w2.y andalso w1.direction = w2.direction
  end operator
  operator TCWord.cast as string
    return chr(x, y, direction, found) & value & hint
  end operator
  mdListDeclare(TCWord)

  type CROSSWORD.TGrid
    as uinteger w, h, wordcount, char(any, any), used(any, any), locked(any, any), empty
    as integer possibilities, fSizeGrid = FONTSIZE.SMALL, fSizeHint = FONTSIZE.TINY
    as mdList(TCWord) word, filling
    as mdList(TWord) missing
    declare constructor
    declare constructor(w_ as uinteger, h_ as uinteger)
    declare sub create(wordlist as mdList(TListPoint), byref count_ as uinteger = -1, minLen as integer = 4, _
                       possibilities as integer = DirecOptions.all)
    declare sub add(x as integer, y as integer, direc as integer, wordPtr as wstring ptr, size as integer)
    declare sub exportHtml(filename as string, headline as string, type_ as integer, title as string = "")
    declare sub fill overload(list as mdList(TWord))
    declare sub fill(txt as TWString)
    declare sub fillRandomly
  end type
end namespace

globalExercises.add(TExercise("soup", localize("soupTitle"), "full", 0, _
                   @CROSSWORD.callbackWordsoup, 0, @CROSSWORD.settings, "", "games"))
globalExercises.add(TExercise("crossword", localize("crossTitle"), "full", 0, _
                   @CROSSWORD.callbackCrossword, 0, @CROSSWORD.settings, "", "games"))

#ifndef createFlexion
  ' if there is no function for random flexion, use uninflected word instead
  function createFlexion(w as TWord, inflection as integer) as TWord
    return w
  end function
#endif

scope
  dim as TList crosswordList, fullList
  fullList.listname = "full"
  fullList.load
  with crosswordList
    .listName = "crossword"
    .list.addAll(fullList.list)
    .save
    .list.clear
  end with
  with crosswordList
    .listName = "wordsoup"
    .list.addAll(fullList.list)
    .save
    .list.clear
  end with
  fullList.list.clear
end scope

constructor CROSSWORD.TCWord
end constructor

constructor CROSSWORD.TCWord(x_ as ubyte, y_ as ubyte, d_ as ubyte, v_ as TWord)
  x         = x_
  y         = y_
  direction = d_
  value     = v_
end constructor

constructor CROSSWORD.TGrid
end constructor

constructor CROSSWORD.TGrid(w_ as uinteger, h_ as uinteger)
  w = w_
  h = h_
  redim char(w-1, h-1)
  redim used(w-1, h-1)
  redim locked(w-1, h-1)
  empty = w*h
end constructor

sub CROSSWORD.TGrid.add(x as integer, y as integer, direc as integer, wordPtr as wstring ptr, size as integer)
  dim as integer ret = 0, dx, dy, spx, spy, px = x, py = y, i = 0
  select case direc
    case Direction.rightup   : dx =  1 : dy = -1 : spx = 1 : spy =  1
    case Direction.right     : dx =  1 : dy =  0 : spx = 0 : spy =  1
    case Direction.rightdown : dx =  1 : dy =  1 : spx = 1 : spy = -1
    case Direction.down      : dx =  0 : dy =  1 : spx = 1 : spy =  0
    case Direction.leftdown  : dx = -1 : dy =  1 : spx = 1 : spy =  1
    case Direction.left      : dx = -1 : dy =  0 : spx = 0 : spy =  1
    case Direction.leftup    : dx = -1 : dy = -1 : spx = 1 : spy = -1
    case Direction.up        : dx =  0 : dy = -1 : spx = 1 : spy =  0
  end select
  for i as integer = 0 to size-1
    if char(px, py) = 0 then empty -= 1
    char(px, py) = asc(wordPtr[i])
    used(px, py) = -1
    if i > 0 and i < size-1 then
      locked(px, py) or= 1 shl direc
      locked(px, py) or= 1 shl ((direc + 4) mod 8)
    end if
    if possibilities and DirecOptions.spaces then
      if px+spx >= lbound(char, 1) and py+spy >= lbound(char, 2) and _
         px+spx <= ubound(char, 1) and py+spy <= ubound(char, 2) then
        locked(px+spx, py+spy) or= 1 shl direc
        locked(px+spx, py+spy) or= 1 shl ((direc + 4) mod 8)
      end if
      if px-spx >= lbound(char, 1) and py-spy >= lbound(char, 2) and _
         px-spx <= ubound(char, 1) and py-spy <= ubound(char, 2) then
        locked(px-spx, py-spy) or= 1 shl direc
        locked(px-spx, py-spy) or= 1 shl ((direc + 4) mod 8)
      end if
    end if
    px += dx
    py += dy
  next
  if possibilities and DirecOptions.numbers _
                   andalso px >= 0 and py >= 0 and px <= ubound(char, 1) andalso py <= ubound(char, 2) _
                   andalso char(px, py) = 0 then
     locked(px, py) = 255
  end if
end sub

sub CROSSWORD.TGrid.create(wordlist as mdList(TListPoint), byref count_ as uinteger = -1, minLen as integer = 4, _
                           possib as integer = DirecOptions.all)
  wordcount = wordlist.size
  possibilities = possib and not DirecOptions.connect
  if count_ = 0 or wordcount = 0 then exit sub
  dim as integer tempLen, tempInt, direc, x, y, rand, posible, best = -1, xMin, xMax, yMin, yMax, flex, tempSize = 0
  dim as TWString fillpattern
  dim as mdList(TWord) tempList
  dim tempWord as TWord, tempCWord as TCword
  dim as TWString backward
  dim as TCWord bestTry, tmpCWord
  dim as integer modification = MODIFY_TOUPPER
  #ifdef MODIFY_NOACUTE
    modification or= MODIFY_NOACUTE
  #endif
  #ifdef MODIFY_NOSTRESS
    modification or= MODIFY_NOSTRESS
  #endif
  ' prepare list of words
  for i as integer = 0 to wordcount-1
    tempWord = globalList.get(wordlist.get(i).id)
    if len(tempWord.map.get("root")) then continue for
    dim as string tempArr()
    if possibilities and DirecOptions.randomFlex then
      tempWord = createFlexion(tempWord, flex)
      tempWord.mark = flex
    else
      tempWord.mark = 0
    end if
    tempWord.word = modifyTxt(tempWord.word, modification)
    for i as integer = 1 to len(tempWord.word)
      if instr(EXTEND_UPPER, tempWord.word.mid(i, 1)) = 0 then continue for, for
    next
    tempWord.word = modifyTxt(tempWord.word, modification)
    if len(tempWord.word) < minLen then continue for
    ' add word (randomly mixed)
    if possibilities and DirecOptions.numbers then tempWord.word = wchr(31) & tempWord.word
    rand = int(rnd*(tempSize+1))
    if rand = tempSize then
      tempList.add(tempWord)
    else
      tempList.add(rand, tempWord)
    end if
    tempSize += 1
  next
  ' add words
  do while count_ > 0
    best = -1
    tempWord = tempList.get(0)
    tempList.remove(0)
    ' exclude words that are part of other words
    for i as integer = backward.wordlength to 1 step -1
      backward &= tempWord.word[i]
    next
    foreach(TCWord, tmpCWord, word)
      if instr(tmpCWord.value.word, tempWord.word) or instr(tempWord.word, tmpCWord.value.word) then continue do
      if instr(tmpCWord.value.word, backward) or instr(backward, tmpCWord.value.word) then continue do
    nexteach
    ' WSTRING of the word
    tempLen = wordLength(tempWord.word)
    if tempLen = 0 then exit do
    ' choose a position in the grid
    if tempLen <= w and tempLen <= h then
      for direc = 0 to Direction.last-1
        select case possibilities and DirecOptions.lastDirec
          case DirecOptions.noDiag
            if direc <> Direction.left and direc <> Direction.right _
               and direc <> Direction.up and direc <> Direction.down then continue for
          case DirecOptions.forwards
            if direc <> Direction.right and direc <> Direction.down then continue for
        end select
        if w < tempLen andalso direc <> Direction.up and direc <> Direction.down then continue for
        if h < tempLen andalso direc <> Direction.left and direc <> Direction.right then continue for
        getMinMaxPos xMin, xMax, yMin, yMax, w, h, tempLen, direc
        for x as integer = xMin to xMax
          for y as integer = yMin to yMax
            posible = checkPosibility(char(), locked(), x, y, direc, possibilities, tempWord.word.value)
            if posible >= 0 then posible = 100*posible + int(rnd*999)
            if posible > best then
              bestTry = TCWord(x, y, direc, TWord)
              best = posible
            end if
          next
        next
      next
      if best >= 0 then
        if possibilities and DirecOptions.numbers then tempWord.word.value[0] = word.size + 1
        if possibilities and DirecOptions.randomFlex then
          tempWord.translate.add(0, flexionLang2(tempWord, tempWord.mark, int(rnd*tempWord.translate.size)))
        end if
        tempCWord = TCWord(bestTry.x, bestTry.y, bestTry.direction, tempWord)
        tempCWord.hint = tempWord.translate.get(0)
        add bestTry.x, bestTry.y, bestTry.direction, tempWord.word.value, tempLen
        word.add tempCWord
        possibilities = possib
        count_ -= 1
      else
        missing.add tempWord
      end if
    end if
  loop
end sub

sub CROSSWORD.TGrid.exportHtml(filename as string, headline as string, type_ as integer, title as string = "")
  ' export grid to a HTML file
  dim as TCWord tempWord
  dim as string tempString
  dim as TWString letter
  dim as integer size, filenumber = freefile
  open filename for output as filenumber
  print #filenumber, "<!DOCTYPE html>"
  print #filenumber, !"<html>\n"
  print #filenumber, "<head>"
  print #filenumber, "<title>Lengua - " & iif(len(title), title, headline) & "</title>"
  print #filenumber, "<meta http-equiv=""content-type"" content=""text/html;charset=utf-8"" />"
  print #filenumber, "<style>"
  print #filenumber, "  td.g  {border:1px black solid; text-align:center; width:2em; height:2em;}"
  if type_ = 1 then
    print #filenumber, "  td.g2 {border:1px black solid; text-align:center; width:2em; height:2em;"
    print #filenumber, "         background-color: #f0f0f0;}"
  end if
  print #filenumber, "</style>"
  print #filenumber, !"</head>\n"
  print #filenumber, "<body>"
  print #filenumber, "  <h1>" & headline & "</h1>"
  print #filenumber, "  <table style=""border-collapse:collapse;"">"
  for y as integer = 0 to h-1
    print #filenumber, !"  <tr>\n    ";
    for x as integer = 0 to w-1
      tempString = "<td class=""g"">"
      if type_ = 1 then
        if char(x, y) > 0 and char(x, y) < 32 then
          letter = str(char(x, y))
        else
          letter = " "
        end if
        if char(x, y) = 0 then tempString = "<td>" else tempString = "<td class=""g2"">"
      else
        letter = char(x, y)
      end if
      print #filenumber, tempString & str(letter) & "</td>";
    next
    if y = 0 then
      print #filenumber, !"\n    <td rowspan=""" & h+1 & !""" style=""padding-left:5em;"">\n      ";
      if type_ = 1 then
        print #filenumber, "<b>" & localize("horizontal") & ":</b><br />"
        for i as integer = 0 to word.size-1
          tempWord = word.get(i)
          if tempWord.direction = Direction.right then
            print #filenumber, i+1 & ") " & tempWord.hint & "<br />"
          end if
        next
        print #filenumber, !"<br />\n<b>" & localize("vertical") & ":</b><br />"
        for i as integer = 0 to word.size-1
          tempWord = word.get(i)
          if tempWord.direction = Direction.down then
            print #filenumber, i+1 & ") " & tempWord.hint & "<br />"
          end if
        next
        print #filenumber, !"\n    </td>";
      elseif type_ = 2 then
        foreach(TCWord, tempWord, word)
          print #filenumber, tempWord.hint & "<br />";
        nexteach
      end if
      print #filenumber, !"\n    </td>";
    end if
    print #filenumber, !"\n  </tr>"
  next
  print #filenumber, "   <tr><td>&nbsp;</td></tr>"
  print #filenumber, "  </table>"
  print #filenumber, "</body>"
  print #filenumber, "</html>"
  close filenumber
end sub

sub CROSSWORD.TGrid.fill(list as mdList(TWord))
  ' fill empty spaces with words from the list
  if list.isEmpty then exit sub
  dim as integer tempInt = 0, tempLen, length(list.size-1)
  dim tempWord as TWord, fillpattern as string
  dim as integer modification = MODIFY_TOUPPER
  #ifdef MODIFY_NOACUTE
    modification or= MODIFY_NOACUTE
  #endif
  #ifdef MODIFY_NOSTRESS
    modification or= MODIFY_NOSTRESS
  #endif
  foreach(TWord, tempWord, list)
    length(tempInt) = wordLength(tempWord.word)
    tempInt += 1
  nexteach
  fillpattern = findFilling(length(), empty)
  dim as integer i = 0
  tempLen = 0
  tempInt = 1
  for y as integer = 0 to h-1
    for x as integer = 0 to w-1
      if char(x, y) = 0 then
        ' check whether the next word has to be choosen
        if tempInt > tempLen then
          i += 1
          if i >= len(fillpattern) then exit for, for
          tempWord = list.get(fillpattern[i])
          filling.add(TCWord(x, y, -1, tempWord))
          list.remove(tempWord)
          tempWord.word = modifyTxt(tempWord.word, modification)
          tempLen = wordLength(tempWord.word)
          tempInt = 1
        end if
        ' insert next char into grid
        char(x, y) = tempWord.word.value[tempInt-1]
        tempInt += 1
      end if
    next
  next
end sub

sub CROSSWORD.TGrid.fill(txt as TWString)
  ' fill empty spaces with given text
  dim as integer i = 0
  for y as integer = 0 to h-1
    for x as integer = 0 to w-1
      if i >= len(txt) then exit for, for
      if char(x, y) = 0 then
        char(x, y) = txt[i]
        i += 1
      end if
    next
  next
end sub

sub CROSSWORD.TGrid.fillRandomly
  ' fill empty spaces with random letters
  dim as TWString txt = string(empty+1, 0), letters = EXTEND_UPPER
  dim as integer size = len(letters)
  for i as integer = 0 to empty-1
    txt.value[i] = letters[int(rnd*size)]
  next
  fill txt
end sub

function CROSSWORD.checkPosibility(char() as integer, locked() as integer, x as integer, y as integer, direc as integer, _
                                   possibilities as integer, txt as wstring ptr) as integer
  dim as integer ret = 0, dx, dy, px = x, py = y, i = 0, connected = 0
  ' if boxes with numbers are shown, reserve first line and first row
  if possibilities and DirecOptions.numbers then
    if y = 0 and direc <> Direction.down then return -1
    if x = 0 and direc <> Direction.right then return -1
  end if
  select case direc
    case Direction.rightup   : dx =  1 : dy = -1 : ret += 1
    case Direction.right     : dx =  1 : dy =  0 : ret += 2
    case Direction.rightdown : dx =  1 : dy =  1 : ret += 1
    case Direction.down      : dx =  0 : dy =  1 : ret += 2
    case Direction.leftdown  : dx = -1 : dy =  1
    case Direction.left      : dx = -1 : dy =  0
    case Direction.leftup    : dx = -1 : dy = -1
    case Direction.up        : dx =  0 : dy = -1
  end select
  do while i < len(*txt)
    if px < lbound(char, 1) or py < lbound(char, 2) or px > ubound(char, 1) or py > ubound(char, 2) then return -1
    if locked(px, py) and (1 shl direc) then return -1
    if char(px, py) > 0 then
      if char(px, py) = asc(txt[i]) then
        ret += 100
        connected = -1
      else
        return -1
      end if
    end if
    i += 1
    px += dx
    py += dy
  loop
  if possibilities and DirecOptions.connect andalso connected = 0 then return -1
  if possibilities and DirecOptions.numbers _
     andalso px >= lbound(char, 1) and py >= lbound(char, 2) and px <= ubound(char, 1) and py <= ubound(char, 2) _
     andalso char(px, py) > 32 then return -1
  return ret
end function

function CROSSWORD.findFilling(length() as integer, empty as integer, first as integer = 0) as string
  dim as string try, bestTry = chr(empty)
  dim as integer last = iif(ubound(length) > 255, 255, ubound(length)), best = 0
  for i as integer = first to last
    if length(i) > empty then continue for
    if length(i) = empty then return chr(0, i)
    try = findFilling(length(), empty-length(i), i+1)
    if try[0] = 0 then return try & chr(i)
    if try[0] < bestTry[0] then bestTry = try & chr(i)
  next
  return bestTry
end function

sub CROSSWORD.getMinMaxPos(byref xMin as integer, byref xMax as integer, byref yMin as integer, byref yMax as integer, _
                 w as integer, h as integer, size as integer, direc as integer)
  select case direc
    case Direction.leftup, Direction.left, Direction.leftdown    : xMin = size-1
    case else                                                    : xMin = 0
  end select
  select case direc
    case Direction.rightup, Direction.right, Direction.rightdown : xMax = w-size
    case else                                                    : xMax = w-1
  end select
  select case direc
    case Direction.leftup, Direction.up, Direction.rightup       : yMin = size-1
    case else                                                    : yMin = 0
  end select
  select case direc
    case Direction.leftdown, Direction.down, Direction.rightdown : yMax = h-size
    case else                                                    : yMax = h-1
  end select
end sub

sub CROSSWORD.getRandomPos(byref x as integer, byref y as integer, w as integer, h as integer, size as integer, direc as integer)
  dim as integer xMin, xMax, yMin, yMax
  getMinMaxPos xMin, xMax, yMin, yMax, w, h, size, direc
  x = int(rnd*(xMax-xMin+1)) + xMin
  y = int(rnd*(yMax-yMin+1)) + yMin
end sub

function CROSSWORD.callbackCrossword(exercise as TExercise) as long
  static as integer showTranslations = -1
  enum
    posNone = 0, posSetting, posHelp, posKeyboard, posShow, posSolve, posNew, posExport, posExit, _
    posHints1, posHints2, posKeys
  end enum
  dim as integer HBx = 20, HBy = 575, LBx = 50, LBy = 580, LBw = 110, LBh = 30, LBp = 20
  dim as integer startX = 40, startY = 100, boxWidth = 28, wordLen, tempInt, allowedInput
  dim as integer mouseX, mouseY, mouseB, mouseWheel, mouseSaveWheel, scrollPos = 0
  dim as boolean mouseInHints, showKeyboard
  dim as integer hintX = 480, hintY = 100, hintW = 300, hintH = 420, transDist = 20, gridW = 15, gridH = 15
  dim as uinteger colGray = &hafafaf, colMark = &hffff77
  dim as string key, tempString
  dim as TWString iKey
  dim as TCoord coord
  dim crosswordGui as TGui, event as TGuiEvent, hintBox1 as TGuiElement, hintBox2 as TGuiElement

  dim as crosswordData guiData
  guiData.posKeys = posKeys
  #ifdef crosswordIncGui
    crosswordIncGui crosswordGui, guiData
    crosswordGui.addElement _
        TGuiElement(GuiElementType.button, posKeyboard, _
                    exepath & "/img/keyboard.bmp", LBx+(LBw+LBp)*5, HBy, 70, 40, chr(9),, "", GuiState.noBorder)
  #else
    guiData.keyboardHeight = 0
    guiData.showDefault = false
    LBx += 70
  #endif
  showKeyboard = guiData.showDefault
  dim as TGuiElement guiKey(guiData.keys.wordlength-1)
  for i as integer = 0 to ubound(guiKey)
    guiKey(i) = crosswordGui.getElement(guiData.posKeys + i)
  next

  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posHelp, _
                  exepath & "/img/help.bmp", HBx, HBy, 40, 40, chr(255, 59),, "", GuiState.noBorder)
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posSetting, _
                  exepath & "/img/settings.bmp", HBx+40, HBy, 40, 40, chr(9),, "", GuiState.noBorder)
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posSolve, LBx+(LBw+LBp), LBy, LBw, LBh, localize("solution"))
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posNew, LBx+(LBw+LBp)*2, LBy, LBw, LBh, localize("newPuzzle"))
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posExport, LBx+(LBw+LBp)*3, LBy, LBw, LBh, localize("exportHTML"))
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posExit, LBx+(LBw+LBp)*4, LBy, LBw, LBh, localize("back"), chr(27))
  hintBox1 = TGuiElement(GuiElementType.textfield, posHints1, hintX, hintY, hintW, hintH, "")
  hintBox2 = TGuiElement(GuiElementType.textfield, posHints2, hintX, hintY, hintW, hintH - guiData.keyboardHeight, "")
  crosswordGui.addElement hintBox1
  crosswordGui.addElement hintBox2

  do
    dim as integer gx = -1, gy, gxTemp, gyTemp, infX = -1, infY
    dim as integer direc, length, wLength, entry(gridW, gridH), toFind, solved
    dim as TTextBlock hints
    ' create crossword puzzle
    messageWait localize("pleaseWait") 
    dim as TGrid grid = TGrid(gridW, gridH)
    dim as TCWord wordlist()
    grid.create exercise.currentList.list, 20,, DirecOptions.forwards or DirecOptions.numbers _
                                                or DirecOptions.connect or DirecOptions.spaces _
                                                or DirecOptions.randomFlex
    grid.word.toArray(wordlist())
    messageWait
    ' create hints
    hints.add hintW, localize("horizontal") & ":", font, grid.fSizeHint
    for i as integer = 0 to ubound(wordlist)
      if wordlist(i).direction = Direction.right then
        hints.add hintW, str(i+1) & ") " & wordlist(i).hint, font, grid.fSizeHint
      end if
    next
    hints.add hintW, "<br />" & localize("vertical") & ":", font, grid.fSizeHint
    for i as integer = 0 to ubound(wordlist)
      if wordlist(i).direction = Direction.down then
        hints.add hintW, str(i+1) & ") " & wordlist(i).hint, font, grid.fSizeHint
      end if
    next
    hints.getImg
    imagedestroy hintBox1.img
    imagedestroy hintBox2.img
    hintBox1.h = hintH
    hintBox2.h = hintH - guiData.keyboardHeight
    hintBox1.img = imagecreate(hints.img->width, hints.img->height)
    hintBox2.img = imagecreate(hints.img->width, hints.img->height)
    put hintbox1.img, (0, 0), hints.img, pset
    put hintbox2.img, (0, 0), hints.img, pset
    if showKeyboard then
      hintBox1.state = GuiState.hidden
      hintBox2.state = GuiState.active
    else
      hintBox1.state = GuiState.active
      hintBox2.state = GuiState.hidden
    end if
    crosswordGui.setElement hintBox1
    crosswordGui.setElement hintBox2
    ' show
    screenlock
    cls
    printFT 30, 30, localize("crossDesc", localize("langFlex1")), font
    line (hintX, hintY)-step(hintW, hintH), &hffffff, bf
    crosswordGui.show
    toFind = 0
    solved = 0
    for y as integer = 0 to grid.h-1
      for x as integer = 0 to grid.w-1
        select case grid.char(x, y)
          case is > 32
            line (x*boxWidth+startX, y*boxWidth+startY)-step(boxWidth, boxWidth),, b
            toFind += 1
          case 1 to 31
            line (x*boxWidth+startX, y*boxWidth+startY)-step(boxWidth, boxWidth),, b
            line (x*boxWidth+startX+1, y*boxWidth+startY+1)-step(boxWidth-2, boxWidth-2), colGray, bf
            centerFT (x+.5)*boxWidth+startX-1, (y+.5)*boxWidth+startY+2, str(grid.char(x, y)), font
          case else
            line (x*boxWidth+startX, y*boxWidth+startY)-step(boxWidth, boxWidth), 0, bf
        end select
      next
    next
    screenunlock

    ' solve
    do
      do
        getmouse mouseX, mouseY, mouseWheel, mouseB
        sleep 1
      loop until mouseB < 1
      do
        key = inkey
        getmouse mouseX, mouseY, mouseWheel, mouseB
        event = crosswordGui.getEvent(key, 0)
        sleep 1
      loop until mouseB > 0 orelse len(key)
      if key = chr(255, 107) then exit do
      iKey = checkInput("", 1,, key, event.id, guiData.posKeys, guiData.keys)
      allowedInput = 0
      if len(iKey) = 0 then iKey = key
      ' get letter position
      if mouseB then
        gxTemp = (mouseX-startX) \ boxWidth
        gyTemp = (mouseY-startY) \ boxWidth
        if gxTemp >= 0 and gyTemp >= 0 and gxTemp < grid.w and gyTemp < grid.h then
          if grid.char(gxTemp, gyTemp) > 0 then
            if gx >= 0 then
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), &hffffff, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
            end if
            gx = gxTemp
            gy = gyTemp
            if grid.char(gxTemp, gyTemp) < 32 then
              direc = wordlist(grid.char(gxTemp, gyTemp)-1).direction
              wLength = wordLength(wordlist(grid.char(gxTemp, gyTemp)-1).value.word) - 2
              length = wLength
              if infX >= 0 then
                line (startX + infX*boxWidth + 1, startY + infY*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colGray, bf
                centerFT startX + (infX+.5)*boxWidth - 1, startY + (infY+.5)*boxWidth + 2, _
                         str(grid.char(infX, infY)), font
              end if
              infX = gx
              infY = gy
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colMark, bf
              centerFT startX + (infX+.5)*boxWidth - 1, startY + (infY+.5)*boxWidth + 2, _
                       str(grid.char(infX, infY)), font
              select case direc
                case direction.right : gx += 1
                case direction.down  : gy += 1
              end select
            else
              if infX >= 0 then
                line (startX + infX*boxWidth + 1, startY + infY*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colGray, bf
                centerFT startX + (infX+.5)*boxWidth - 1, startY + (infY+.5)*boxWidth + 2, _
                         str(grid.char(infX, infY)), font
              end if
              infX = -1
              direc = 0
              length = 0
            end if
            line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colMark, bf
            centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
          end if
        end if
      end if

      ' input
      if gx >= 0 andalso len(iKey) then
        select case key
          case chr(8)
            if entry(gx, gy) = 0 andalso direc andalso length < wLength then
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), &hffffff, bf
              select case direc
                case direction.right : gx -= 1
                case direction.down  : gy -= 1
              end select
              if entry(gx, gy) = grid.char(gx, gy) then toFind += 1
              entry(gx, gy) = 0
              length += 1
            else
              if entry(gx, gy) = grid.char(gx, gy) then toFind += 1
              entry(gx, gy) = 0
            end if
          case " " 
            if entry(gx, gy) = grid.char(gx, gy) then toFind += 1
            entry(gx, gy) = 0
          case chr(255, 72)
            if grid.char(gx, gy-1) > 31 then
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), &hffffff, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
              gy -= 1
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colMark, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
              if direc = direction.down then
                length += 1
              else
                if infX >= 0 then
                  line (startX + infX*boxWidth + 1, startY + infY*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colGray, bf
                  centerFT startX + (infX+.5)*boxWidth - 1, startY + (infY+.5)*boxWidth + 2, _
                           str(grid.char(infX, infY)), font
                  infX = -1
                end if
                length = 0
                direc = 0
              end if
            end if
          case chr(255, 75)
            if grid.char(gx-1, gy) > 31 then
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), &hffffff, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
              gx -= 1
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colMark, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
              if direc = direction.right then
                length += 1
              else
                if infX >= 0 then
                  line (startX + infX*boxWidth + 1, startY + infY*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colGray, bf
                  centerFT startX + (infX+.5)*boxWidth - 1, startY + (infY+.5)*boxWidth + 2, _
                           str(grid.char(infX, infY)), font
                  infX = -1
                end if
                length = 0
                direc = 0
              end if
            end if
          case chr(255, 77)
            if grid.char(gx+1, gy) > 31 then
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), &hffffff, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
              gx += 1
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colMark, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
              if direc = direction.right then
                length -= 1
              else
                if infX >= 0 then
                  line (startX + infX*boxWidth + 1, startY + infY*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colGray, bf
                  centerFT startX + (infX+.5)*boxWidth - 1, startY + (infY+.5)*boxWidth + 2, _
                           str(grid.char(infX, infY)), font
                  infX = -1
                end if
                length = 0
                direc = 0
              end if
            end if
          case chr(255, 80)
            if grid.char(gx, gy+1) > 31 then
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), &hffffff, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
              gy += 1
              line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colMark, bf
              centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
              if direc = direction.down then
                length -= 1
              else
                if infX >= 0 then
                  line (startX + infX*boxWidth + 1, startY + infY*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colGray, bf
                  centerFT startX + (infX+.5)*boxWidth - 1, startY + (infY+.5)*boxWidth + 2, _
                           str(grid.char(infX, infY)), font
                  infX = -1
                end if
                length = 0
                direc = 0
              end if
            end if
          case else
            dim as integer iUni = iKey[0], xUni
            for i as integer = 0 to len(EXTEND_UPPER)-1
              xUni = EXTEND_UPPER[i]
              if xUni = iUni or xUni = unicode2upper(iUni) then
                allowedInput = true
                if entry(gx, gy) = grid.char(gx, gy) then toFind += 1
                entry(gx, gy) = xUni
                if entry(gx, gy) = grid.char(gx, gy) then toFind -= 1
                exit for
              end if
            next
          end select
        screenlock
        if length > 0 andalso allowedInput then
          line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), &hffffff, bf
          centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
          select case direc
            case direction.right : gx += 1
            case direction.down  : gy += 1
          end select
          length -= 1
        end if
        line (startX + gx*boxWidth + 1, startY + gy*boxWidth + 1)-step(boxWidth-2, boxWidth-2), colMark, bf
        centerFT startX + (gx+.5)*boxWidth - 1, startY + (gy+.5)*boxWidth + 2, entry(gx, gy), font
        screenunlock
      end if

      ' check GUI
      select case event.id
        case posHelp
          tempInt = showHelpScreen(MODULE_PATH & "doc.htm", "crossword")
          if tempInt = RETVAR.windowClosed then key = chr(255, 107)
        case posSetting
          dim as FB.Image ptr tempImg = imagecreate(SCREEN_X, SCREEN_Y)
          get (0, 0)-(SCREEN_X-1, SCREEN_Y-1), tempImg
          tempInt = settings(exercise)
          if tempInt = RETVAR.windowClosed then key = chr(255, 107)
          put (0, 0), tempImg, pset
          imagedestroy tempImg
        case posShow
          showTranslations = not showTranslations
        case posNew
          if toFind <= 0 orelse message(localize("askQuitPuzzle"), localize("yes|no"), 2) = 1 then
            continue do, do
          end if
        case posSolve
          screenlock
          for x as integer = 0 to grid.w - 1
            for y as integer = 0 to grid.h - 1
              if grid.char(x, y) > 32 and grid.char(x, y) <> entry(x, y) then
                entry(x, y) = grid.char(x, y)
                line (startX + x*boxWidth + 1, startY + y*boxWidth + 1)-step(boxWidth-2, boxWidth-2), &hffffff, bf
                centerFT (x+.5)*boxWidth+startX-1, (y+.5)*boxWidth+startY+2, grid.char(x, y), font,, &hdf0000
                toFind = 0
                solved = -1
              end if
            next
          next
          screenunlock
        case posExport
          grid.exportHtml(exepath & "/export/" & LANGUAGE & "/crossword.htm", localize("crossTitle1"), 1)
          message(localize("exportFileName", localize("crossArticle"), "export/" & LANGUAGE & "/crossword.htm"))
        case posKeyboard
          screenlock
          line (hintX, hintY)-step(hintW, hintH), &hffffff, bf
          showKeyboard = not showKeyboard
          for i as integer = 0 to ubound(guiKey)
            guiKey(i).state = iif(showKeyboard, GuiState.active, GuiState.hidden)
            crosswordGui.setElement(guiKey(i))
            guiKey(i).show(,true)
          next
          if showKeyboard then
            hintBox1.state = GuiState.hidden
            hintBox2.state = GuiState.active
            crosswordGui.setElement hintBox1
            crosswordGui.setElement hintBox2
            hintBox2.show
          else
            hintBox1.state = GuiState.active
            hintBox2.state = GuiState.hidden
            crosswordGui.setElement hintBox1
            crosswordGui.setElement hintBox2
            hintBox1.show
          end if
          screenunlock
        case posExit
          if toFind <= 0 orelse message(localize("askQuit"), localize("yes|no"), 2) = 1 then exit do
      end select
      if toFind = 0 and solved = 0 then
        message(localize("puzzleSolved"))
        solved = -1
      end if
    loop
    if key = chr(255, 107) then return RETVAR.windowClosed
    return RETVAR.ok
  loop
end function

function CROSSWORD.callbackWordsoup(exercise as TExercise) as long
  static as integer showTranslations = -1
  enum
    posNone = 0, posSetting, posHelp, posShow, posSolve, posNew, posExport, posExit
  end enum
  dim as integer LBx = 120, LBy = 580, LBw = 110, LBh = 30, LBp = 20
  dim as integer startX = 40, startY = 100, boxWidth = 38, mouseX, mouseY, mouseB, direc, wordLen, oldX, oldY, toFind, tempInt
  dim as integer transStartX = 480, transStartY = 100, transW = 300, transH = 380, transDist = 20
  dim as uinteger gx1, gx2, gy1, gy2, colGrey = &h9f9f9f
  dim key as string, tempString as string, coord as TCoord
  dim crosswordGui as TGui, event as TGuiEvent
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posHelp, _
                  exepath & "/img/help.bmp", LBx-100, LBy, 40, 40, chr(255, 59),, "", GuiState.noBorder)
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posSetting, _
                  exepath & "/img/settings.bmp", LBx-60, LBy, 40, 40, chr(9),, "", GuiState.noBorder)
  dim as TGuiElement buttonShow = TGuiElement(GuiElementType.button, posShow, LBx, LBy, LBw, LBh, _
         localize("display", iif(showTranslations, localize("off"), localize("on"))))
  crosswordGui.addElement buttonShow
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posSolve, LBx+(LBw+LBp), LBy, LBw, LBh, localize("solution"))
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posNew, LBx+(LBw+LBp)*2, LBy, LBw, LBh, localize("newPuzzle"))
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posExport, LBx+(LBw+LBp)*3, LBy, LBw, LBh, localize("exportHTML"))
  crosswordGui.addElement _
      TGuiElement(GuiElementType.button, posExit, LBx+(LBw+LBp)*4, LBy, LBw, LBh, localize("back"), chr(27))
  
  do
    ' create wordsoup puzzle
    dim as FB.Image ptr transList = imagecreate(transW, transH+transDist, &hffffff)
    dim as TGrid grid = TGrid(10, 10)
    dim as TCWord wordlist()
    grid.create exercise.currentList.list,,, DirecOptions.all
    grid.fill grid.missing
    grid.fillRandomly
    grid.word.toArray(wordlist())
    toFind = grid.word.size
    for i as integer = 0 to ubound(wordlist)
      printFT iif(i < transH\transDist, 0, transW\2), _
              iif(i < transH\transDist, i, (i-transH\transDist))*20, _
              wordlist(i).hint, font, FONTSIZE.X_SMALL,, transList
    next
    screenlock
    cls
    printFT 30, 30, localize("soupDesc1", localize("langFlex1")), font
    crosswordGui.show
    buttonShow.show    ' the button label may differ from the label in the gui
    select case grid.filling.size
      case 0
      case 1
        printFT 30, 50, localize("soupDesc2", localize("oneMoreWord")), font
      case else
        printFT 30, 50, localize("soupDesc2", localize("moreWords", str(grid.filling.size))), font
    end select
    for y as integer = 0 to grid.h-1
      for x as integer = 0 to grid.w-1
        line (x*boxWidth+startX, y*boxWidth+startY)-step(boxWidth, boxWidth),, b
        centerFT (x+.5)*boxWidth+startX-1, (y+.5)*boxWidth+startY+2, grid.char(x, y), font
      next
    next
    if showTranslations then put (transStartX, transStartY), transList, pset
    screenunlock

    ' solve
    do
      oldX = -1
      ' start and end position of the word
      do
        getmouse mouseX, mouseY,, mouseB
        sleep 1
      loop until mouseB < 1
      ' mark words in the grid
      do
        getmouse mouseX, mouseY,, mouseB
        key = inkey
        sleep 1
      loop until mouseB = 1 orelse len(key)
      if key = chr(255, 107) then exit do
      gx1 = (mouseX-startX) \ boxWidth
      gy1 = (mouseY-startY) \ boxWidth
      ' check GUI
      event = crosswordGui.getEvent(key, 0)
      select case event.id
        case posHelp
          tempInt = showHelpScreen(MODULE_PATH & "doc.htm", "wordsoup")
          if tempInt = RETVAR.windowClosed then key = chr(255, 107)
        case posSetting
          dim as FB.Image ptr tempImg = imagecreate(SCREEN_X, SCREEN_Y)
          get (0, 0)-(SCREEN_X-1, SCREEN_Y-1), tempImg
          tempInt = settings(exercise)
          if tempInt = RETVAR.windowClosed then key = chr(255, 107)
          put (0, 0), tempImg, pset
          imagedestroy tempImg
        case posShow
          showTranslations = not showTranslations
          if showtranslations then
            put (transStartX, transStartY), transList, pset
            buttonShow.title = localize("display", localize("off"))
            buttonShow.show
          else
            line (transStartX, transStartY)-step(transW-1, transH-1), &hffffff, bf
            buttonShow.title = localize("display", localize("on"))
            buttonShow.show
          end if
        case posNew
          if toFind <= 0 orelse message(localize("askQuitPuzzle"), localize("yes|no"), 2) = 1 then
            imagedestroy transList
            continue do, do
          end if
        case posSolve
          if toFind >= 0 then
            for i as integer = 0 to ubound(wordlist)
              if wordlist(i).found = 0 then
                dim as integer dx = 0, dy = 0, size = wordLength(wordlist(i).value.word)-1
                select case wordlist(i).direction
                  case DIRECTION.leftup, DIRECTION.left, DIRECTION.leftdown
                    dx = -size
                  case DIRECTION.rightup, DIRECTION.right, DIRECTION.rightdown
                    dx = size
                end select
                select case wordlist(i).direction
                  case DIRECTION.leftup, DIRECTION.up, DIRECTION.rightup
                    dy = -size
                  case DIRECTION.leftdown, DIRECTION.down, DIRECTION.rightdown
                    dy = size
                end select
                line ((wordlist(i).x+.5)*boxWidth+startX, (wordlist(i).y+.5)*boxWidth+startY)-step(dx*boxWidth, dy*boxWidth)
                wordlist(i).found = -1
              end if
            next
            toFind = -1
            dim as FB.Image ptr yellow = imagecreate(boxWidth-1, boxWidth-1, &h000077)
            dim tempWord as TCWord, countLetters as integer = 0
            foreach(TCWord, tempWord, grid.filling)
              countLetters += wordLength(tempWord.value.word)
            nexteach
            for y as integer = 0 to grid.w-1
              for x as integer = 0 to grid.h-1
                if countLetters < 1 then exit for, for
                if grid.used(x, y) = 0 then
                  put (startX+x*boxWidth+1, startY+y*boxWidth+1), yellow, xor
                  countLetters -= 1
                end if
              next
            next
            imagedestroy yellow
          end if
        case posExport
          grid.exportHtml(exepath & "/export/" & LANGUAGE & "/wordsoup.htm", localize("soupTitle1"), 2)
          message(localize("exportFileName", localize("soupArticle"), "export/" & LANGUAGE & "/wordsoup.htm"))
        case posExit
           if toFind <= 0 orelse message(localize("askQuit"), localize("yes|no"), 2) = 1 then exit do
      end select    
      if gx1 > grid.w-1 or gy1 > grid.h-1 then continue do
      ' release button in grid
      do
        getmouse mouseX, mouseY,, mouseB
        key = inkey
        if key = chr(255, 107) orelse _
                 (key = chr(27) andalso (toFind <= 0 orelse message(localize("askQuit"), localize("yes|no"), 2) = 1)) then exit do, do
        gx2 = (mouseX-startX) \ boxWidth
        gy2 = (mouseY-startY) \ boxWidth
        if gx2 <> oldX or gy2 <> oldY then
          if oldX >= 0 then mark gx1, oldX, gy1, oldY, startX, startY, boxWidth
          if gx2 < grid.w and gy2 < grid.h then
            mark gx1, gx2, gy1, gy2, startX, startY, boxWidth
            oldX = gx2
            oldY = gy2
          else
            oldX = -1
          end if
        end if
        sleep 1
      loop until mouseB < 1
      if oldX >= 0 then mark gx1, oldX, gy1, oldY, startX, startY, boxWidth
      if gx2 > grid.w-1 or gy2 > grid.h-1 then continue do
      ' check if a correct word was found
      if gy2 = gy1 then
        direc = iif(gx2 > gx1, Direction.right, Direction.left)
        wordLen = abs(gx2-gx1)+1
      elseif gx2 = gx1 then
        direc = iif(gy2 > gy1, Direction.down, Direction.up)
        wordLen = abs(gy2-gy1)+1
      elseif gx2-gx1 = gy2-gy1 then
        direc = iif(gx2 > gx1, Direction.rightdown, Direction.leftup)
        wordLen = abs(gx2-gx1)+1
      elseif gx2-gx1 = gy1-gy2 then
        direc = iif(gx2 > gx1, Direction.rightup, Direction.leftdown)
        wordLen = abs(gx2-gx1)+1
      else
        direc = -1
      end if
      if direc <> -1 then
        dim as integer found = -1
        for i as integer = 0 to ubound(wordlist)
          if gx1 = wordlist(i).x and gy1 = wordlist(i).y then
            if wordList(i).direction = direc andalso wordLength(wordlist(i).value.word) = wordLen then
              found = i
              exit for
            end if
          elseif gx2 = wordlist(i).x and gy2 = wordlist(i).y then
            if wordList(i).direction = (direc+4) mod 8 andalso wordLength(wordlist(i).value.word) = wordLen then
              found = i
              exit for
            end if
          end if
        next
        if found >= 0 andalso wordlist(found).found = 0 then
          ' mark found word in the grid
          dim as integer x = gx1, y = gy1
          dim as FB.Image ptr grey = imagecreate(boxWidth-1, boxWidth-1, &hf7f7f7)
          for i as integer = 0 to wordLength(wordList(found).value.word)-1
            put (x*boxWidth+startX+1, y*boxWidth+startY+1), grey, and
            x += sgn(gx2-gx1)
            y += sgn(gy2-gy1)
          next
          imagedestroy grey
          line ((gx1+.5)*boxWidth+startX, (gy1+.5)*boxWidth+startY)-((gx2+.5)*boxWidth+startX, (gy2+.5)*boxWidth+startY)
          wordlist(found).found = -1
          ' rewrite translation list
          line transList, (0, 0)-step(transW-1, transH+transDist-1), &hffffff, bf
          for i as integer = 0 to ubound(wordList)
            tempString = wordlist(i).hint
            coord = FTsize(tempString, font, FONTSIZE.X_SMALL)
            printFT iif(i < transH\transDist, 0, transW\2), _
                    iif(i < transH\transDist, i, (i-transH\transDist))*20, _
                    wordlist(i).hint, font, FONTSIZE.X_SMALL, iif(wordlist(i).found, colGrey, 0), transList
            if wordlist(i).found then
              line transList, (iif(i < transH\transDist, 0, transW\2), (i mod (transH\transDist))*20+10)-step(coord.x+2, 0), colGrey
            end if
          next
          ' mark translation of current word
          if showTranslations then
            coord = FTsize(wordList(found).hint, font, FONTSIZE.X_SMALL)
            dim as FB.Image ptr markImg = imagecreate(coord.x+2, coord.y-5, &h555555)
            put (transStartX, transStartY), transList, pset
            ' mark translation of the found word
            line markImg, (0, 0)-step(coord.x+1, coord.y-6), &h000077, bf
            put (transStartX+iif(found < transH\transDist, 0, transW\2), transStartY+(found mod (transH\transDist))*20), markImg, xor
            imagedestroy markImg
          end if
          toFind -= 1
        end if
      end if
    loop
    imagedestroy transList
    if key = chr(255, 107) then return RETVAR.windowClosed
    return RETVAR.ok
  loop
end function

sub CROSSWORD.mark(gx1 as integer, gx2 as integer, gy1 as integer, gy2 as integer, _
                   startX as integer, startY as integer, boxWidth as integer)
  dim as FB.Image ptr grey = imagecreate(boxWidth-1, boxWidth-1, &h333333)
  if gx2 = gx1 then
    for y as integer = gy1 to gy2 step sgn(gy2-gy1+.1)
      put (gx1*boxWidth+startX+1, y*boxWidth+startY+1), grey, xor
    next
  elseif gy2 = gy1 then
    for x as integer = gx1 to gx2 step sgn(gx2-gx1+.1)
      put (x*boxWidth+startX+1, gy1*boxWidth+startY+1), grey, xor
    next
  elseif abs(gx2-gx1) = abs(gy2-gy1) then
    for x as integer = 0 to abs(gx2-gx1)
      put ((gx1+x*sgn(gx2-gx1))*boxWidth+startX+1, (gy1+x*sgn(gy2-gy1))*boxWidth+startY+1), grey, xor
    next
  end if
  imagedestroy grey
end sub

function CROSSWORD.settings(exercise as TExercise, optional as long = SETTING_OPTIONS.NONE) as long
  return chooseWords(exercise, exercise.settingString, optional or SETTING_OPTIONS.SHOWTAGS)
end function
