namespace TRAINCONJ
  declare function callback(exercise as TExercise) as long
  declare function settings(exercise as TExercise, optional as long = SETTING_OPTIONS.NONE) as long
  declare sub drawAll(list as mdList(TListPoint), allPages as TPages, page as integer, gui as TGui)
end namespace

globalExercises.add(TExercise("trainconj", "Konjugations-Trainer", "trainconj", 0, _
                    @TRAINCONJ.callback, 0, @TRAINCONJ.settings,, "grammar"))

function TRAINCONJ.callback(exercise as TExercise) as long
  ' vocabulary test (spaced repetition with SuperMemo algorithm SM-2)
  
  ' initialize list of verbs
  if exercise.fullList.list.isEmpty then
    message "Es konnten keine|Verben gefunden werden.",, 1
    return RETVAR.emptyList
  end if
  if exercise.currentList.list.isEmpty then
    exercise.currentList.list.addAll exercise.fullList.list
  end if
  if exercise.currentList.list.get(0).id = 0 then
    message "Es konnten keine Verben gefunden werden. Bitte|rufen zuerst das Konfigurationsmenü auf und|" _
            "wählen Sie die Verben aus, die Sie prüfen wollen.",, 1
    return RETVAR.emptyList
  end if

  dim as integer MAXLENGTH = 35
  dim as string german
  dim listpoint as TListPoint, w as TWord
  dim as integer count = 10, mx, my, mb, randomValue, randomField = exercise.currentList.size\5 + 1
  dim as integer randomTense, randomPerson, randomNum, cursor, repeat = 0
  dim as integer txtX = 80, hl1Y = 80, hl2Y = 120, inpY = 200, solY = 240, tabSize = 90
  dim as integer progX = 200, progY = 610, progW = 400, progH = 10
  dim as string wRoot, solution, key
  dim as FB.Image ptr okImg = imagecreate(16, 16)
  bload exepath & "/img/ok.bmp", okImg

  dim as string conjTxt(...) = _
                { "Presente (Indicativo)", "Imperfecto (Indicativo)",       "Indefinido (Indicativo)", _
                  "Perfecto (Indicativo)", "Pluscuamperfecto (Indicativo)", "Pret. Anterior", _
                  "Presente Subjuntivo",   "Imperfecto Subjuntivo",         "", _
                  "Perfecto Subjuntivo",   "Pluscuamperfecto Subjuntivo",   "Imperativo", _
                  "Futuro (Indicativo)",   "Futuro Subjuntivo",             "Futuro Perfecto (Indicativo)", _
                  "Futuro Perfecto Subjuntivo", "Condicional", "Condicional Perfecto" }
  dim as integer conjID(...) = _
                { INFLECT.PRES,                    INFLECT.PRETIMP,                 INFLECT.PRETIND, _
                  INFLECT.PERF,                    INFLECT.PQPERF,                  INFLECT.PERFANT, _
                  INFLECT.PRES or INFLECT.SUBJ,    INFLECT.PRETIMP or INFLECT.SUBJ, 0, _
                  INFLECT.PERF or INFLECT.SUBJ,    INFLECT.PQPERF  or INFLECT.SUBJ, INFLECT.IMPERAT, _
                  INFLECT.FUTSIMP,                 INFLECT.FUTSIMP or INFLECT.SUBJ, INFLECT.FUTCOMP, _
                  INFLECT.FUTCOMP or INFLECT.SUBJ, INFLECT.COND,                    INFLECT.CONDPERF }
  dim as string conjSetting = mid(exercise.settingString, 33, ubound(conjTxt)+1)
  if conjSetting = string(ubound(conjID)+1, 0) then
    for i as integer = 0 to ubound(conjTxt)
      if len(conjTxt(i)) then conjSetting[i] = 1
    next
  end if

  ' prepare GUI
  enum
    posNone = 0, posRepeat, posShow, posTable, posHelp, posExit
    posKeys = 20
  end enum
  dim as TGui testGui
  dim as TGuiElement tempButton, repeatButton, showButton, tabButton
  dim as TInput inpt = TInput(txtX, inpY, FONTSIZE.NORMAL, localize("input") & ": ", "", -1, 435)
  dim as TGuiEvent event
  ' special keys
  guiAddKeys testGui, posKeys, SPECIAL_KEYS, 80, 350, 30, 30, 15, 15, 11
  ' button: repeat verb
  repeatButton = TGuiElement(GuiElementType.button, posRepeat, 80, 530, 120, 50, "gleiches Verb", chr(9),, "Tab")
  repeatButton.fontsize = FONTSIZE.SMALL
  testGui.addElement repeatButton
  ' button: repeat verb
  tabButton = TGuiElement(GuiElementType.button, posTable, _
              600, inpY, 120, 30, "zeige Tabelle", chr(255, 60),, "F2", GuiState.hidden)
  testGui.addElement tabButton
  ' button: exit
  tempButton = TGuiElement(GuiElementType.button, posExit, 600, 530, 120, 50, "Abbruch")
  tempButton.fontsize = FONTSIZE.SMALL
  testGui.addElement tempButton
  ' button: show correct answer
  showButton = TGuiElement(GuiElementType.button, posShow, 330, 530, 120, 50, "ANZEIGEN", chr(13),, "Enter")
  showButton.fontsize = FONTSIZE.SMALL
  testGui.addElement showButton
  ' button: show program help
  testGui.addElement _
      TGuiElement(GuiElementType.button, posHelp, _
                  exepath & "/img/help.bmp", 690, 320, 40, 40, chr(255, 59),, "F1", GuiState.noBorder)
  
  dim as uinteger barCol(2)  = { rgb(  0, 255, 0), rgb(255, 0, 0), rgb(255, 255, 255) }
  dim as integer progress(2) = { 0, 0, count }

  do
    if repeat = 0 then
      randomValue = int(rnd*randomField)
      listPoint = exercise.currentList.list.get(randomValue)
      w = globallist.get(listPoint.id)
      if len(w.word) = 0 then
        continue do
      end if
      wRoot = w.map.get("root")
      if len(wRoot) = 0 then wRoot = w.word
    end if
    do
      randomTense = int(rnd*(ubound(conjTxt)+1))
    loop until conjSetting[randomTense]
    randomPerson = int(rnd*3)+1
    randomNum = int(rnd*2)
    if conjID(randomTense) = INFLECT.IMPERAT and randomPerson = 1 and randomNum = 0 then randomPerson = 2
    solution = conjugSpanish(w, (randomPerson + randomNum*INFLECT.PLURAL) or conjID(randomTense)).word
    german   = conjugGerman (w, (randomPerson + randomNum*INFLECT.PLURAL) or conjID(randomTense))

    inpt.value = ""
    repeatButton.state = GuiState.deactivated
    tabButton.state = GuiState.hidden
    testGui.setElement tabButton
    screenlock
    cls
    testGui.show
    repeatButton.show
    drawProgress progX, progY, progW, progH, progress(), barCol()
    centerFT SCREEN_X\2, 50, "Konjugationstrainer", font, 20
    printFT txtX, hl1Y, "Bilden Sie die korrekte Konjugationsform von:", font
    printFT txtX, hl2Y, w.word & " - " & german, font
    inpt.show
    drawProgress progX, progY, progW, progH, progress(), barCol()
    screenunlock

    if repeat = 0 then
      exercise.currentList.list.add listPoint
      exercise.currentList.list.remove randomValue
    end if
    do
      getmouse mx, my,, mb
      sleep 1
    loop until mb < 1
    do
      key = inkey
      event = testGui.getEvent(key)
      if len(key) orelse event.id then
        inpt.check(key, event.id, posKeys, SPECIAL_KEYS)
        inpt.show
      end if
      select case event.id
        case posHelp
          dim as integer ret = showHelpScreen(MODULE_PATH & "doc.htm", "trainconj")
          if ret = RETVAR.windowClosed then key = chr(255, 107)
        case posExit
          select case message(localize("askQuitExerc"), localize("yes|no"), 2)
            case 1
              key = chr(27)
              exit do, do
            case RETVAR.windowClosed
              key = chr(255, 107)
          end select
      end select
      sleep 1
    loop until event.id = posShow or key = chr(255, 107)
    if key = chr(255, 107) then exit do
    progress(2) -= 1
    screenlock
    line (txtX, solY - 5)-step(400, 0)
    printFT txtX, solY, "Lösung:", font
    printFT txtX + tabSize, solY, solution, font
    if solution = inpt.value then
      put (txtX - 20, solY), okImg, pset
      progress(0) += 1
    else
      progress(1) += 1
    end if
    inpt.show
    drawProgress progX, progY, progW, progH, progress(), barCol()
    screenunlock

    repeatButton.state = GuiState.active
    tabButton.state    = GuiState.active
    showButton.title = "weiter"
    repeatButton.show
    testGui.setElement tabButton
    tabButton.show
    showButton.show
    showButton.title = "ANZEIGEN"
    repeat = 0
    do
      getmouse mx, my,, mb
      sleep 1
    loop until mb < 1
    do
      key = inkey
      event = testGui.getEvent(key)
      select case event.id
        case posHelp
          dim as integer ret = showHelpScreen(MODULE_PATH & "doc.htm", "trainconj")
          if ret = RETVAR.windowClosed then key = chr(255, 107)
        case posRepeat
          repeat = -1
          if progress(2) < 1 then progress(2) += 1
          exit do
        case posTable
          dim as FB.Image ptr img = imagecreate(SCREEN_X, SCREEN_Y)
          get (0, 0)-(SCREEN_X-1, SCREEN_Y-1), img
          wordInfo(w)
          put (0, 0), img, pset
          imagedestroy img
        case posExit
          if progress(2) < 1 then event.id = posShow : exit do
          select case message(localize("askQuitExerc"), localize("yes|no"), 2)
            case 1
              key = chr(27)
              exit do, do
            case RETVAR.windowClosed
              key = chr(255, 107)
          end select
      end select
      sleep 1
    loop until event.id = posShow or key = chr(255, 107)
  loop until progress(2) < 1 or key = chr(255, 107)

  imagedestroy okImg
  select case key
    case chr(255, 107) : return RETVAR.windowClosed
    case chr(27)       : return RETVAR.aborted
    case else          : return RETVAR.ok
  end select
  return RETVAR.ok
end function

function TRAINCONJ.settings(exercise as TExercise, optional as long = SETTING_OPTIONS.NONE) as long
  ' initialize list of verbs
  exercise.fullList.list.clear
  dim as mdList(string) verbList
  dim as mdList(long) ks = globallist.keyset
  dim as TWord word
  dim as long i, k
  dim as string w, lp
    foreach(long, i, ks)
      word = globallist.get(i)
      if word.groupname = "verb" then
        w = word.map.get("root")
        if len(w) = 0 then w = word.word
        if right(w, 2) = "se" then w = left(w, len(w)-2)
        if verbList.contains(w) = 0 then
          for k = 0 to verbList.size-1
            lp = verbList.get(k)
            if lp > w then verbList.add k, w : exercise.fullList.list.add k, i : exit for
          next
          if k = verbList.size then verbList.add w : exercise.fullList.list.add i
        end if
      end if
    nexteach
  verbList.clear

  enum Index
    ok = 1, help, choose, all, none, conj
  end enum
  dim as string key, verbConj(...) = _
                { "Presente Ind.",      "Imperfecto Ind.",     "Indefinido Ind.", _
                  "Perfecto Ind.",      "Pluscuamperf. Ind.",  "Pret. Anterior", _
                  "Presente Subj.",     "Imperfecto Subj.",    "", _
                  "Perfecto Subj.",     "Pluscuamperf. Subj.", "Imperativo", _
                  "Futuro Ind.",        "Futuro Subj.",        "Futuro Perf. Ind.", _
                  "Futuro Perf. Subj.", "Condicional",         "Condicional Perf." }
  dim as string choosenVerbs = mid(exercise.settingString, ubound(verbConj)+34)
  dim as integer vXStart = 20, vYStart = 150, vWidth = 119, vHeight = 30, vMod = 6, vXPlus = 10, vYPlus = 10
  dim as integer okX     = 640, okY     = 500, okW     = 120, okH     = 30
  dim as integer helpX   = 340, helpY   = 495
  dim as integer chooseX = 490, chooseY = 500, chooseW = 120, chooseH = 30
  dim as integer allX    =  40, allY    = 500, allW    = 120, allH    = 30
  dim as integer noneX   = 190, noneY   = 500, noneW   = 120, noneH   = 30
  dim as integer mx, my, mb

  dim as TGuiElement conjButton(ubound(verbConj))
  if len(exercise.settingString) < ubound(verbConj)+33 then
    exercise.settingString &= string(ubound(verbConj)+33-len(exercise.settingString), 0)
  end if
  chooseSingleWords exercise, choosenVerbs, SETTING_OPTIONS.SKIPDIALOG

  dim settingGui as TGui, event as TGuiEvent

  if optional and SETTING_OPTIONS.SKIPDIALOG then
    for i as integer = 0 to ubound(verbConj)
      if exercise.settingString[i+32] > 0 then conjButton(i).state = GuiState.activated
    next
  else
    cls
    printFT vXStart, vYStart-50, "Verwende folgende Konjugationen:", font, 16
    for i as integer = 0 to ubound(verbConj)
      if len(verbConj(i)) then
        conjButton(i) = TGuiElement(GuiElementType.button, Index.conj+i, vXstart+(i mod vMod)*(vWidth+vXplus), _
                        vYStart+(i\vMod)*(vHeight+vYplus), vWidth, vHeight, verbConj(i))
        if exercise.settingString[i+32] > 0 then conjButton(i).state = GuiState.activated
        settingGui.addElement conjButton(i)
      end if
    next
    dim as TGuiElement okButton     = TGuiElement(GuiElementType.button, Index.ok, okX, okY, okW, okH, "zurück", chr(27))
    dim as TGuiElement helpButton   = _
        TGuiElement(GuiElementType.button, Index.help, _
                    exepath & "/img/help.bmp", helpX, helpY, 40, 40, chr(255, 59),,, GuiState.noBorder)
    dim as TGuiElement chooseButton = _
        TGuiElement(GuiElementType.button, Index.choose, chooseX, chooseY, chooseW, chooseH, _
                    "Wörter wählen", chr(255, 60))
    dim as TGuiElement allButton = _
        TGuiElement(GuiElementType.button, Index.all, allX, allY, allW, allH, "alle markieren")
    dim as TGuiElement noneButton = _
        TGuiElement(GuiElementType.button, Index.none, noneX, noneY, noneW, noneH, "keine markieren")
    settingGui.addElement okButton
    settingGui.addElement helpButton
    settingGui.addElement chooseButton
    settingGui.addElement allButton
    settingGui.addElement noneButton
    settingGui.show
    do
      getmouse mx, my,, mb
      sleep 1
    loop until mb < 1
    do
      key = inkey
      event = settingGui.getEvent(key)
      if event.mbutton = 1 orelse len(event.key) then
        select case event.id
          case Index.none
            for i as integer = 0 to ubound(verbConj)
              conjButton(i).state = GuiState.active
              settingGui.setElement conjButton(i)
            next
            settingGui.show
          case Index.all
            for i as integer = 0 to ubound(verbConj)
              conjButton(i).state = GuiState.activated
              settingGui.setElement conjButton(i)
            next
            settingGui.show
          case Index.choose
            dim as FB.Image ptr tempImg = imagecreate(SCREEN_X, SCREEN_Y)
            get (0, 0)-step(SCREEN_X-1, SCREEN_Y-1), tempImg
            dim as integer ret = chooseSingleWords(exercise, choosenVerbs)
            put (0, 0), tempImg, pset
            imagedestroy tempImg
            if ret = RETVAR.windowClosed then key = chr(255, 107) : exit do
          case Index.help
            dim as integer ret = showHelpScreen(MODULE_PATH & "doc.htm", "optionsTrainConj")
            if ret = RETVAR.windowClosed then key = chr(255, 107) : exit do
          case Index.conj to Index.conj+ubound(verbConj)
            if conjButton(event.id - Index.conj).state = GuiState.active then
              conjButton(event.id - Index.conj).state = GuiState.activated
            else
              conjButton(event.id - Index.conj).state = GuiState.active
            end if
            settinggui.setElement conjButton(event.id - Index.conj)
            conjButton(event.id - Index.conj).show
        end select
      end if
      do
        getmouse mx, my,, mb
        sleep 1
      loop until mb < 1
      if event.id = Index.ok and (key = chr(27) or checkMouseIn(okX, okY, okW, okH) <> 0) then exit do
    loop until key = chr(255, 107)
  end if

  ' create exercise.settingString
  exercise.settingString = string(ubound(verbConj)+33, 0) & choosenVerbs
  for i as integer = 0 to ubound(verbConj)
    if conjButton(i).state = GuiState.activated then exercise.settingString[i+32] = 1
  next
  exercise.currentList.mix
  if key = chr(255, 107) then return RETVAR.windowClosed
  return RETVAR.ok
end function

sub TRAINCONJ.drawAll(list as mdList(TListPoint), allPages as TPages, page as integer, gui as TGui)
  screenlock
  cls
  allPages.printEntries(list, page)
  gui.show
  screenunlock
end sub
