' basic module for language training Russian-German

#include once "include/TWString.bi"
const SCREEN_X = 800, SCREEN_Y = 640, B_COUNT = 6
const moduleVersionMajor = 2, moduleVersionMinor = 4, moduleVersionPatch = 180112
const LANGTITLE1 = "Russisch",   LANGTITLE2 = "Deutsch"
const LANGSHORT1 = "ru",         LANGSHORT2 = "de"

dim shared as string LANGUAGE, MODULE_PATH, WORDS_VERSION
LANGUAGE = mid(__PATH__, instrrev(__PATH__, any "/\")+1)
MODULE_PATH = exepath & "/modules/" & LANGUAGE & "/"

' TODO: nearly everything ...

' options to modify text to a different form
' BE SURE NOT TO USE CONFLICTING OPTIONS!
const MODIFY_TOLOWER       =   1, _ ' change to lower case if possible
      MODIFY_TOUPPER       =   2, _ ' change to upper case if possible
      MODIFY_NOPARANTHESES =   4, _ ' omit parantheses ( and ) 
      MODIFY_NOSPECIAL     =   8, _ ' omit everything exept Russian and German letters
      MODIFY_NOUMLAUT      =  32, _ ' change ß to ss (or SS if MODIFY_TOUPPER), Ä to Ae (or AE or ae) and Ё to E
      MODIFY_NONUMBERS     =  64, _ ' omit 0-9
      MODIFY_SORTABLE      = 128, _ ' various changes like lower case, ä->ae ...
      MODIFY_NOSTRESS      = 256, _ ' omit escape string for stress
      MODIFY_ALPHANUM      = 512    ' everthing exept a-z, A-Z and 0-9 is changed to an underline character

' inflection forms
namespace INFLECT
  const PERS1 = 1, PERS2 = 2, PERS3 = 3
  const SINGULAR = 0, PLURAL = 3
  const INF = 0, PRES = 8, PRET = 16, FUT = 24, IMPERAT = 32, REFLEX = 40
  const MASC = 0, FEM = 1, GENUS = 1, SING = 0, PL = 2, NUM = 2, ADVERB = 4
  const FULL = 1, MINIMUM = 2
end namespace

const COLOR_GRAY = rgb(127, 127, 127)

const FONT_STRESS_ESCAPE = 95
const FONT_STRESS_CHAR   = 95
#include once "include/fontExt.bi"
dim shared font as FT_FaceFam

#include once "include/globalFunctions.bi"
dim shared as TExercise globalExercArray(), globalOptArray()

declare function conjugRussian(verb as TWord, inflection as integer) as TWord
declare function conjugGerman(verb as TWord, inflection as integer, no as integer = 0) as string
declare function declinRussian(noun as TWord, inflection as integer) as string
declare function declinGerman(noun as TWord, inflection as integer, no as integer = 0) as string
declare function firstUpper(txt as string) as string
declare function flexionCount(w as TWord, opt as integer = INFLECT.FULL) as integer
declare function getArticle(w as TWord, both as integer = 0) as string
declare sub init(mainModule as TModule ptr)
declare function modifyTxt(wtxt as TWString, opt as integer = MODIFY_TOLOWER or MODIFY_NOPARANTHESES) as string
declare function number(first as integer = 0, last as integer = 0) as TWord
declare sub speak(txt as string, no as integer = 1, byref ready as integer = 0)
declare sub speakThread(txt as string, no as integer = 1)
declare sub wordInfo(byval w as TWord)

#include once "specific.bi"
#include once "include/common/lang-basic.bi"
#include once "addGerman.bi"

#include once "include/common/editWords.bi"
#include once "include/common/learning.bi"
#include once "include/common/wordtest.bi"
#include once "include/common/diagram.bi"
#include once "include/common/dictionary.bi"
#include once "incgui.bi"

'#include once "dictionary.bi"
'#include once "export.bi"

' games
#include once "include/common/pairs.bi"
#include once "include/common/crossword.bi"
#include once "include/common/hangman.bi"
globalExercises.add(TExercise("games", localize("gamesTitle"), "full", MENUTYPE.exercTwoRowsExt, _
                    @createCategory, 0, 0, localize("gamesHeadline")))

function conjugRussian(verb as TWord, inflection as integer) as TWord
  return verb
end function

function declinRussian(noun as TWord, inflection as integer) as string
  ' not implemented yet
  return noun.word
end function

function firstUpper(txt as string) as string
  if asc(txt) < 128 then
    return ucase(left(txt, 1)) & mid(txt, 2)
  else
    select case left(txt, 2)
      case "ä", "ö", "ü", "á", "é", "í", "ó", "ú", "ñ"
        return chr(txt[0]) & chr(txt[1]-32) & mid(txt, 3)
    end select
  end if
  return txt
end function

function flexionCount(w as TWord, opt as integer = INFLECT.FULL) as integer
  select case w.groupname
    case "verb"
      select case opt
        case INFLECT.MINIMUM : return 6
        case else            : return 6
      end select
    case "noun"      : return 3
    case "adjective" : return 4
    case else        : return -1
  end select
end function

function getArticle(w as TWord, both as integer = 0) as string
  return " "
end function

sub init(mainModule as TModule ptr) export
  initModule(mainModule)
  dim as TList fullList
  fullList.listname = "complete"
  fullList.add , 0, &hffffff
  fullList.save
  fullList.listname = "full"
  fullList.del 0, &hffffff, "group", "cardinal"
  fullList.save
  initDefaultLists(fullList)
  mainWindow
  exitModule
end sub

function infoDefault(exercise as TExercise) as string
  ' default information string for exercises (shown in the main window)
  dim as TList ptr listPtr = iif(exercise.currentList.size = 0, @exercise.fullList, @exercise.currentList)
  return "ausstehend: " & listPtr->notChecked & " von " & listPtr->size & " Wörtern"
end function

function infoEmpty(exercise as TExercise) as string
  ' empty information string for exercises (for the main window)
  return ""
end function

function modifyTxt(wtxt as TWString, opt as integer = MODIFY_TOLOWER or MODIFY_NOPARANTHESES) as string
  ' modify input (see modify options at the top of the file)
  ' Using conflicting options may produce unexpected results
  dim as TWString ret
  for i as integer = 0 to len(wtxt) - 1
    if wtxt[i] > 127 andalso opt and MODIFY_ALPHANUM then ret &= "_" : continue for
    if wtxt[i] = FONT_STRESS_ESCAPE andalso _
       opt and (MODIFY_NOSTRESS or MODIFY_NOSPECIAL or MODIFY_ALPHANUM) then continue for
    select case as const wtxt[i]
      case 0 to 31                                    ' ignore control chars
      case 65 to 90                                   ' upper case
        ret &= chr(wtxt[i] + iif(opt and (MODIFY_TOLOWER or MODIFY_SORTABLE), 32, 0))
      case 97 to 122                                  ' lower case
        ret &= chr(wtxt[i] - iif(opt and MODIFY_TOUPPER, 32, 0))
      case 40, 41                                     ' parentheses
        if opt and MODIFY_ALPHANUM then
          ret &= "_"
        else
          if opt and (MODIFY_NOPARANTHESES or MODIFY_NOSPECIAL) = 0 then ret &= chr(wtxt[i])
        end if
      case 48 to 57                                   ' numbers
        if (opt and MODIFY_NONUMBERS) = 0 then ret &= chr(wtxt[i])
      case 32 to 39, 42 to 47, 58 to 64, 91 to 96, 123 to 127   ' special keys
        if opt and MODIFY_ALPHANUM then
          ret &= "_"
        else
          if (opt and MODIFY_NOSPECIAL) = 0 then ret &= chr(wtxt[i])
        end if
      case 196 ' Ä
        if opt and MODIFY_SORTABLE or MODIFY_NOUMLAUT then
          if opt and (MODIFY_SORTABLE or MODIFY_TOLOWER) then
            ret &= "ae"
          elseif opt and MODIFY_TOUPPER then
            ret &= "AE"
          else
            ret &= "Ae"
          end if
        else
          ret &= iif(opt and MODIFY_TOLOWER, "ä", "Ä")
        end if
      case 228 ' ä
        if opt and MODIFY_SORTABLE or MODIFY_NOUMLAUT then
          ret &= iif(opt and MODIFY_TOUPPER, "AE", "ae")
        else
          ret &= iif(opt and MODIFY_TOUPPER, "Ä", "ä")
        end if
      case 214 ' Ö
        if opt and MODIFY_SORTABLE or MODIFY_NOUMLAUT then
          if opt and (MODIFY_SORTABLE or MODIFY_TOLOWER) then
            ret &= "oe"
          elseif opt and MODIFY_TOUPPER then
            ret &= "OE"
          else
            ret &= "Oe"
          end if
        else
          ret &= iif(opt and MODIFY_TOLOWER, "ö", "Ö")
        end if
      case 246 ' ö
        if opt and MODIFY_SORTABLE or MODIFY_NOUMLAUT then
          ret &= iif(opt and MODIFY_TOUPPER, "OE", "oe")
        else
          ret &= iif(opt and MODIFY_TOUPPER, "Ö", "ö")
        end if
      case 220 ' Ü
        if opt and MODIFY_SORTABLE or MODIFY_NOUMLAUT then
          if opt and (MODIFY_SORTABLE or MODIFY_TOLOWER) then
            ret &= "ue"
          elseif opt and MODIFY_TOUPPER then
            ret &= "UE"
          else
            ret &= "Ue"
          end if
        else
          ret &= iif(opt and MODIFY_TOLOWER, "ü", "Ü")
        end if
      case 252 ' ü
        if opt and MODIFY_SORTABLE or MODIFY_NOUMLAUT then
          ret &= iif(opt and MODIFY_TOUPPER, "UE", "ue")
        else
          ret &= iif(opt and MODIFY_TOUPPER, "Ü", "ü")
        end if
      case 159 ' ß
        if opt and (MODIFY_SORTABLE or MODIFY_NOUMLAUT) then
          ret &= iif(opt and MODIFY_TOUPPER, "SS", "ss")
        else
          ret &= "ß"
        end if
      case 1040 to 1071 ' А ... Я
        ret &= TWString(iif(opt and (MODIFY_TOLOWER or MODIFY_SORTABLE), wtxt[i]+32, wtxt[i]))
      case 1072 to 1103 ' а ... я
        ret &= TWString(iif(opt and MODIFY_TOUPPER, wtxt[i]-32, wtxt[i]))
      case 1025 ' Ё
        if opt and MODIFY_SORTABLE then
          ret &= iif(opt and MODIFY_TOLOWER, "eё", "Eё")
        elseif opt and MODIFY_NOUMLAUT then
          ret &= iif(opt and MODIFY_TOLOWER, "e", "E")
        else
          ret &= "Ё"
        end if
      case 1105 ' ё
        if opt and MODIFY_SORTABLE then
          ret &= iif(opt and MODIFY_TOUPPER, "Eё", "eё")
        elseif opt and MODIFY_NOUMLAUT then
          ret &= iif(opt and MODIFY_TOUPPER, "E", "e")
        else
          ret &= "ё"
        end if
    end select
  next
  return ret
end function

sub speak(txt as string, no as integer = 1, byref ready as integer = 0)
  ' TODO (or not ...)

  'mutexlock speakLock
  'dim search as integer, t as string = txt
  'ready = -1
  'mutexunlock speakLock
  'search = instr(t, "ll")
  'do while search
  '  t = left(t, search-1) & "y" & mid(t, search+2)
  '  search = instr(t, "ll")
  'loop
  'search = instr(t, "rr")
  'do while search
  '  t = left(t, search) & "'" & mid(t, search+1)
  '  search = instr(t, "rr")
  'loop
  'shell "espeak -vmb-es" & no & " """ & t & """ -s100"
end sub

sub speakThread(txt as string, no as integer = 1)
' see speak()
'  dim as integer ready = 0, r
'  dim as any ptr thread = threadcall speak(txt, no, ready)
'  do
'    mutexlock speakLock
'    r = ready
'    mutexunlock speakLock
'  loop until r
'  threaddetach(thread)
end sub

sub wordInfo(byval w as TWord)
end sub
